@@ -11,12 +11,16 @@ from .base import *
 from .imageio import *
 
 import numpy as np
-import zbarlight
 import skimage as ski
 from skimage.color import rgb2lab
 import piexif
 from math import log2
 
+try:
+    import zbarlight
+except ImportError:
+    pass  # fail quietly for now. warn if actually used
+
 
 class ImageMeanColourException(Exception):
     """Thrown if image has an unknown pixel matrix shape (i.e. not greyscale or colour)"""
@@ -60,7 +64,10 @@ class ScanQRCodesStep(PipelineStep):
     """
     def process_file(self, file):
         assert hasattr(file, "pixels")  # TODO proper check
-        codes = zbarlight.scan_codes('qrcode', file.pil)
+        try:
+            codes = zbarlight.scan_codes('qrcode', file.pil)
+        except NameError:
+            codes = None  # TODO fail less quietly
         if codes is not None:
             codes = ';'.join(sorted(x.decode('utf8') for x in codes))
         file.report.update({"QRCodes": codes})
