@@ -37,6 +37,7 @@ class AbortPipelineForThisImage(Exception):
     """Thrown for errors stopping the pipeline for the current image being processed only."""
     pass
 
+
 def indent(text, by=2):
     res = []
     for line in text.split("\n"):
@@ -48,6 +49,7 @@ class TSPipeline(object):
     """
     Core TimeStream pipeline runner.
     """
+
     def __init__(self, *args, reporter=None):
         r"""
         :param \*args: Pipeline steps to execute
@@ -125,7 +127,7 @@ class TSPipeline(object):
                         self.n += 1
                         yield file
             else:
-                for file in tqdm(input_stream, unit=" files"):
+                for file in input_stream:
                     file = self.process_file(file)
                     if file is None:
                         continue
@@ -134,7 +136,7 @@ class TSPipeline(object):
                     yield file
         except FatalPipelineError as exc:
             print(f"Apologies, we encountered a fatal pipeline error, and are stopping processing. The error is:\n{str(exc)}", file=stderr)
-            self.retcode=1
+            self.retcode = 1
 
     def __call__(self, *args, **kwargs):
         yield from self.process(*args, **kwargs)
@@ -172,6 +174,7 @@ class ResultRecorder(object):
     Saves pipeline data to a TSV file. Usually used with
     :class:`.ResultRecorderStep`, where the file path is specified.
     """
+
     def __init__(self):
         self.fields = []
         self.data = defaultdict(dict)
@@ -217,6 +220,7 @@ class LiveResultRecorder(ResultRecorder):
     (compare with :class:`.ResultRecorder`, which stores results and writes them later in a TSV)
     Unlike :class:`.ResultRecorder`, records file path is determined on init.
     """
+
     def __init__(self, fileorpath):
         """
         :param fileorpath: Writable file handle or path to records file
@@ -325,6 +329,7 @@ class WriteFileStep(PipelineStep):
 
 class FileStatsStep(PipelineStep):
     """Reports file name and size."""
+
     def process_file(self, file):
         file.report.update({"FileName": op.basename(file.filename),
                             "FileSize": len(file.content)})
@@ -333,6 +338,7 @@ class FileStatsStep(PipelineStep):
 
 class FilterStep(PipelineStep):
     """Filter out files from the rest of a pipeline based on a callback function."""
+
     def __init__(self, callback, message="Filter excluded image"):
         """
         :param callback: Callback function that accepts file as argument and returns true to continue the pipeline and false otherwise.
@@ -359,6 +365,7 @@ class ConditionalStep(PipelineStep):
 
     will write only images whose sum of all pixel values is greater than 100.
     """
+
     def __init__(self, conditional_callback, step):
         """
         :param conditional_callback: Callback function that accepts file as argument and returns true to continue the pipeline and false otherwise.
@@ -389,9 +396,47 @@ class ClearFileObjectStep(PipelineStep):
     coordinating thread, which would significantly impact performance and hog
     CPU and memory.
     """
+
     def process_file(self, file):
         file._content = None
         if hasattr(file, "_pixels"):
             file._pixels = None
         gc.collect()
         return file
+
+
+class TimeStreamPathRenameStep(PipelineStep):
+    """A helper step to allow renaming of TimeStreamFiles to a new timestream path filename
+    """
+
+    def __init__(self, name, add_subsecond_field=False, flat_output=False):
+        """
+        :param name: name of timetream to use to rename files
+        :type name: str
+
+        :param add_subsecond_field: Enable for timestreams with sub-second records, using an additional ``_[00-99]`` at the end of filenames
+        :type add_subsecond_field: bool, optional
+        :param flat_output: Store timestream in a flat file structure, instead of Timestream directory structure
+        :type flat_output: bool, optional
+        """
+        self.name = name
+        self.add_subsecond_field = add_subsecond_field
+        self.flat_output = flat_output
+
+    def _timestream_path(self, file):
+        """Gets path for timestream file."""
+        idxstr = ""
+        if file.instant.index is not None:
+            idxstr = "_" + str(file.instant.index)
+        if self.add_subsecond_field:
+            idxstr = "_00" + idxstr
+        fname = f"{self.name}_%Y_%m_%d_%H_%M_%S{idxstr}.{file.format}"
+        if self.flat_output:
+            path = fname
+        else:
+            path = f"%Y/%Y_%m/%Y_%m_%d/%Y_%m_%d_%H/{fname}"
+        return file.instant.datetime.strftime(path)
+
+    def process_file(self, file):
+        file.filename = self._timestream_path(file)
+        return file
