@@ -4,27 +4,25 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import sys
+import io
+import re
+from sys import stderr, stdout, stdin
+import os.path as op
+import datetime as dt
+from .base import PipelineStep, AbortPipelineForThisImage
+from ..timestream import *
+from ..utils import *
+from ..time import *
+from PIL import Image
+from skimage.color import rgb2lab
+import skimage as ski
+import imageio
+import cv2
+import rawpy
 import numpy as np
 import os
 os.environ["OMP_NUM_THREADS"] = "1"  # force single-threaded demosaicing in rawpy
-import rawpy
-import cv2
-import imageio
-import skimage as ski
-from skimage.color import rgb2lab
-from PIL import Image
-
-from ..time import *
-from ..utils import *
-from ..timestream import *
-from .base import PipelineStep, AbortPipelineForThisImage
-
-import datetime as dt
-import os.path as op
-from sys import stderr, stdout, stdin
-import re
-import io
-import sys
 
 
 class DecodeImageFileStep(PipelineStep):
@@ -77,7 +75,16 @@ class DecodeImageFileStep(PipelineStep):
                     else:
                         pixels = img.raw_image.copy()
             else:
+                # use imageio to infer file format
+                if file.format in (None, "", "."):
+
+                    r = imageio.get_reader(file.content)
+                    f = list(filter(lambda x: x.lstrip(".").lower() in self.default_options.keys(),
+                                    r.format.extensions))
+                    if len(f) > 0:
+                        file.format = f[0].lstrip(".").lower()
                 pixels = imageio.imread(file.content)
+
         except Exception as exc:
             raise AbortPipelineForThisImage(f"Error on image read: {exc.__class__.__name__}: {str(exc)}")
         return TimestreamImage.from_timestreamfile(file, pixels=pixels)
@@ -142,6 +149,7 @@ class EncodeImageFileStep(PipelineStep):
                 content = buf.getvalue()
         elif self.format == "png" or self.format == "jpg":
             content = imageio.imwrite('<bytes>', file.rgb_8, **self.options)
+
         # reinstatiate and demote to a TimestreamFile
         return TimestreamFile(content=content, filename=filename,
                               instant=file.instant, report=file.report,
@@ -238,7 +246,7 @@ class TimestreamImage(TimestreamFile):
             "filename": file.filename,
             "fetcher": file.fetcher,
             "report": file.report,
-            "content": file.content,
+            "content": file.content
         }
         params.update(kwargs)
         return cls(**params)
