@@ -3,17 +3,17 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import os.path as op
 import datetime
 import calendar
 import re
-import os.path as op
+import json
 
 
 TS_DATEFMT = "%Y_%m_%d_%H_%M_%S"
 TS_DATETIME_RE = re.compile(r"(\d\d\d\d[._\- ]?\d\d[._\- ]?\d\d[._\- T]?\d\d[._\- :]?\d\d[._\- :]?\d\d)(_\w+)?")
 
 
-
 def parse_date(datestr):
     """Parses dates in iso8601-ish formats to :class:`datetime.datetime` objects
 
@@ -148,6 +148,16 @@ class TSInstant(object):
     def __repr__(self):
         return str(self)
 
+    def dict(self):
+        return {
+            "index": self.index,
+            "datetime": str(self.datetime),
+            "datetime_iso": self.iso8601()
+        }
+
+    def json(self):
+        return json.dumps(self.dict())
+
     def iso8601(self):
         """ Converts own datetime to a ISO-8601 string.
 
@@ -231,6 +241,7 @@ def parse_partial_date(datestr, max=False):
 class TimeFilter(object):
     """ Check datetimes fall within a given time period.
     """
+
     def __init__(self, startdate=None, enddate=None, starttime=None, endtime=None):
         """ Initiates with a datetime range to check against.
 
