@@ -3,8 +3,6 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from tqdm import tqdm
-
 import copy
 import datetime as dt
 import hashlib
@@ -27,6 +25,7 @@ from pyts2.time import *
 from pyts2.utils import *
 from pyts2.filelock import FileLock
 
+
 def get_instant_or_fn(path):
     try:
         return str(TSInstant.from_path(path))
@@ -99,6 +98,12 @@ class Fetcher(object):
         elif obj["type"] == "tar":
             return TarContentFetcher(obj["archivepath"], obj["pathintar"])
 
+    def dict(self):
+        return {"cls": str(self.__class__)}
+
+    def json(self):
+        return json.dumps(self.dict())
+
     @property
     def instant(self):
         """ Gets datetime from file path.
@@ -236,6 +241,7 @@ class TimestreamFile(object):
         self.instant = instant
         self.filename = filename
         self.fetcher = fetcher
+        self.output_filename = None
         if filename is None and fetcher is not None:
             self.filename = fetcher.filename
         self._content = content
@@ -247,8 +253,6 @@ class TimestreamFile(object):
             self.instant = TSInstant.from_path(self.filename)
         if self.instant is None:
             raise ValueError("TimestreamFile must have an instant")
-        if self.filename is None:
-            raise ValueError("TimestreamFile must have a filename")
         if self.__class__ is TimestreamFile and self._content is None and self.fetcher is None:
             raise ValueError("TimestreamFile must have content (directly or via a fetcher)")
         if format is None:
@@ -336,6 +340,19 @@ class TimestreamFile(object):
     def __repr__(self):
         return self.filename
 
+    def dict(self):
+        return {
+            "filename": self.filename,
+            "md5sum": self.md5sum,
+            "format": self.format,
+            "report": self.report,
+            "instant": None if self.instant is None else self.instant.dict(),
+            "fetcher": None if self.fetcher is None else self.fetcher.dict()
+        }
+
+    def json(self, **dumps_kwargs):
+        return json.dumps(self.dict(), **dumps_kwargs)
+
     @property
     def md5sum(self):
         return self.checksum('md5')
@@ -394,7 +411,7 @@ class TimeStream(object):
             raise ValueError("TimeFilter is not valid")
         self.timefilter = timefilter
         if bundle_level not in self.bundle_levels:
-            raise ValueError("invalid bundle level %s",  bundle_level)
+            raise ValueError("invalid bundle level %s", bundle_level)
         self.bundle = bundle_level
         self._index_file = ""
         if path is not None:
@@ -436,7 +453,7 @@ class TimeStream(object):
                     with open(self._index_file, "r") as fh:
                         self._files = {}
                         self._instants = {}
-                        for line in tqdm(fh):
+                        for line in fh:
                             fetcher = Fetcher.from_json(json.loads(line))
                             self._files[fetcher.filename] = fetcher
                             self._instants[fetcher.instant] = fetcher
@@ -521,7 +538,7 @@ class TimeStream(object):
             yield from self._scan_dir(basedir)
 
             inot = inotify.adapters.InotifyTree(basedir)
-            for _ in range(600): # rescan every 10m
+            for _ in range(600):  # rescan every 10m
                 for ev in inot.event_gen(timeout_s=1, yield_nones=False):
                     (_, type_names, path, filename) = ev
                     if 'IN_ISDIR' in type_names or not 'IN_CLOSE_WRITE' in type_names and not 'IN_MOVED_TO' in type_names:
@@ -535,8 +552,6 @@ class TimeStream(object):
             # clean up inotify watcher
             del inot
 
-
-
     def from_fofn(self, pathorfile):
         """ Yields files from a list of file names, most commonly from an index file.
 
@@ -689,12 +704,13 @@ class TimeStream(object):
         if not isinstance(file, TimestreamFile):
             raise TypeError("file should be a TimestreamFile")
         subpath = self._timestream_path(file)
-        if (self.bundle == "none") and (self.path is not None):
-            outpath = op.join(self.path, subpath)
-            os.makedirs(op.dirname(outpath), exist_ok=True)
-            with FileLock(outpath):
-                with open(outpath, 'wb') as fh:
-                    fh.write(file.content)
+        if self.bundle == "none":
+            if self.path is not None:
+                outpath = op.join(self.path, subpath)
+                os.makedirs(op.dirname(outpath), exist_ok=True)
+                with FileLock(outpath):
+                    with open(outpath, 'wb') as fh:
+                        fh.write(file.content)
         else:
             if self.bundle == "root":
                 self.path = str(self.path)
@@ -725,3 +741,97 @@ class TimeStream(object):
         """ Maintains interface as other objects with ``close()`` methods but does not currently need further actions.
         """
         pass
+
+
+class InMemoryTimeStream(TimeStream):
+    """InMemoryTimestream
+    doesnt support bundling yet
+
+    """
+
+    def __init__(self, name=None, format=None,
+                 # bundle_level="none",
+                 timefilter=None,
+                 add_subsecond_field=False, flat_output=False):
+        """
+        :param name: Name of timestream
+        :type name: str
+        :param format: Format of files in timestream, usually the same as the file extension (e.g. ``.jpg``)
+        :type format: str
+        # :param bundle_level: Smallest time unit to bundle files at.
+        # :type bundle_level: str
+        :param timefilter: Timefilter object for defining datetime range to iterate over
+        :type timefilter: :class:`time.Timefilter`, optional
+        :param add_subsecond_field: Enable for timestreams with sub-second records, using an additional ``_[00-99]`` at the end of filenames
+        :type add_subsecond_field: bool, optional
+        :param flat_output: Store timestream in a flat file structure, instead of Timestream directory structure
+        :type flat_output: bool, optional
+        """
+        self.name = name
+        if format is not None:
+            format = format.lstrip(".").lower()
+            if format == "tiff":
+                format = "tif"
+            if format == "jpeg":
+                format = "jpg"
+        self.format = format
+        self.sorted = True
+        self.add_subsecond_field = add_subsecond_field
+        self.flat_output = flat_output
+        if timefilter is not None and not isinstance(timefilter, TimeFilter):
+            raise ValueError("TimeFilter is not valid")
+        self.timefilter = timefilter
+        # if bundle_level not in self.bundle_levels:
+        #     raise ValueError("invalid bundle level %s", bundle_level)
+        # self.bundle = bundle_level
+        # self._bundles = []
+        self._images = []
+
+    def write(self, file):
+        if self.name is None:
+            raise RuntimeError("TSv2Stream not opened")
+        if not isinstance(file, TimestreamFile):
+            raise TypeError("file should be a TimestreamFile")
+        #  set the filename for the timestream file if it doesnt have one.
+        if file.filename in (None, ""):
+            file.filename = self._timestream_path(file)
+        self._images.append(file)
+
+    def iter(self):
+        for file in self._images:
+            if self.timefilter is not None and not self.timefilter(file.instant.datetime):
+                continue
+            yield file
+
+    def from_inotify(self):
+        raise NotImplementedError("from_inotify is not implemented in InMemoryTimestream")
+
+    def from_fofn(self):
+        raise NotImplementedError("from_fofn is not implemented in InMemoryTimestream")
+
+    @property
+    def instants(self):
+        return [f.instant for f in self._images]
+
+    def getinstant(self, value):
+        """ Retrieves files from timestream at a specific time point.
+
+        :param value: Time point to retrieve
+        :type value: :class:`time.TSInstant`
+        :return: File if present
+        :rtype: :class:`.TimestreamFile`
+        """
+
+        if isinstance(value, TimestreamFile):
+            value = value.instant
+        assert(isinstance(value, TSInstant))
+        for f in self._images:
+            if f.instant == value:
+                return f
+        return None
+
+    def index(self):
+        raise NotImplementedError("no indexing functionality exists for InMemoryTimestream")
+
+    def open(self, path, format=None):
+        raise NotImplementedError("opening an InMemoryTimestream is not implemented")
